@@ -1,17 +1,19 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-#from __future__ import absolute_import, division, print_function
+# from __future__ import absolute_import, division, print_function
 
+import locale
 import matplotlib as mpl
 import numpy as np
+import random
 from matplotlib.projections.polar import PolarAxes
 from numpy.lib.twodim_base import histogram2d
 import matplotlib.pyplot as plt
 from pylab import poly_between
 
 RESOLUTION = 100
-ZBASE = -1000 #The starting zorder for all drawing, negative to have the grid on
+ZBASE = -1000  # The starting zorder for all drawing, negative to have the grid on
 VAR_DEFAULT = 'speed'
 DIR_DEFAULT = 'direction'
 FIGSIZE_DEFAULT = (8, 8)
@@ -37,33 +39,36 @@ class WindAxesFactory(object):
         else:
             raise(NotImplementedError("typ=%r but it might be in %s" % (typ, d.keys())))
 
+
 class WindroseAxes(PolarAxes):
     """
 
     Create a windrose axes
 
     """
+    name = 'windrose'
 
     def __init__(self, *args, **kwargs):
         """
         See Axes base class for args and kwargs documentation
         """
-        
-        #Uncomment to have the possibility to change the resolution directly 
-        #when the instance is created
-        #self.RESOLUTION = kwargs.pop('resolution', 100)
+
+        # Uncomment to have the possibility to change the resolution directly
+        # when the instance is created
+        # self.RESOLUTION = kwargs.pop('resolution', 100)
+        self.rmax = kwargs.pop('rmax', None)
         PolarAxes.__init__(self, *args, **kwargs)
         self.set_aspect('equal', adjustable='box', anchor='C')
         self.radii_angle = 67.5
         self.cla()
 
     @staticmethod
-    def from_ax(ax=None, fig=None, *args, **kwargs):
+    def from_ax(ax=None, fig=None, rmax=None, *args, **kwargs):
         if ax is None:
             if fig is None:
                 fig = plt.figure(figsize=FIGSIZE_DEFAULT, dpi=DPI_DEFAULT, facecolor='w', edgecolor='w')
             rect = [0.1, 0.1, 0.8, 0.8]
-            ax = WindroseAxes(fig, rect, axisbg='w', *args, **kwargs)
+            ax = WindroseAxes(fig, rect, facecolor='w', rmax=rmax, *args, **kwargs)
             fig.add_axes(ax)
             return ax
         else:
@@ -79,21 +84,21 @@ class WindroseAxes(PolarAxes):
         self.theta_labels = ['E', 'N-E', 'N', 'N-W', 'W', 'S-W', 'S', 'S-E']
         self.set_thetagrids(angles=self.theta_angles, labels=self.theta_labels)
 
-        self._info = {'dir' : list(),
-                      'bins' : list(),
-                      'table' : list()}
+        self._info = {
+            'dir': list(),
+            'bins': list(),
+            'table': list()
+        }
 
         self.patches_list = list()
 
-
     def _colors(self, cmap, n):
-        '''
+        """
         Returns a list of n colors based on the colormap cmap
 
-        '''
+        """
         return [cmap(i) for i in np.linspace(0.0, 1.0, n)]
 
-
     def set_radii_angle(self, **kwargs):
         """
         Set the radii labels angle
@@ -104,18 +109,25 @@ class WindroseAxes(PolarAxes):
         if angle is None:
             angle = self.radii_angle
         self.radii_angle = angle
-        radii = np.linspace(0.1, self.get_rmax(), 6)
-        radii_labels = [ "%.1f" %r for r in radii ]
-        radii_labels[0] = "" #Removing label 0
-        self.set_rgrids(radii=radii, labels=radii_labels,
-                               angle=self.radii_angle, **kwargs)
-
+        N = 5
+        rmax = self.get_rmax()
+        radii = np.linspace(0, rmax, N + 1)
+        if rmax % N == 0:
+            fmt = "%d"
+        else:
+            fmt = "%.1f"
+        radii_labels = [fmt % r for r in radii]
+        # radii_labels[0] = ""  # Removing label 0
+        self.set_rgrids(radii=radii[1:], labels=radii_labels[1:],
+                        angle=self.radii_angle, **kwargs)
 
     def _update(self):
-        self.set_rmax(rmax=np.max(np.sum(self._info['table'], axis=0)))
+        if self.rmax is None:
+            self.set_rmax(rmax=np.max(np.sum(self._info['table'], axis=0)))
+        else:
+            self.set_rmax(rmax=self.rmax)
         self.set_radii_angle(angle=self.radii_angle)
 
-
     def legend(self, loc='lower left', **kwargs):
         """
         Sets the legend location and her properties.
@@ -149,47 +161,101 @@ class WindroseAxes(PolarAxes):
         handlelen = 0.05     # the length of the legend lines
         handletextsep = 0.02 # the space between the legend line and legend text
         borderaxespad = 0.02       # the border between the axes and legend edge
+        decimal_places = 1   # the decimal places of the formated legend
         """
 
         def get_handles():
             handles = list()
             for p in self.patches_list:
                 if isinstance(p, mpl.patches.Polygon) or \
-                isinstance(p, mpl.patches.Rectangle):
+                        isinstance(p, mpl.patches.Rectangle):
                     color = p.get_facecolor()
                 elif isinstance(p, mpl.lines.Line2D):
                     color = p.get_color()
                 else:
                     raise AttributeError("Can't handle patches")
                 handles.append(mpl.patches.Rectangle((0, 0), 0.2, 0.2,
-                    facecolor=color, edgecolor='black'))
+                               facecolor=color, edgecolor='black'))
             return handles
 
-        def get_labels():
+        def get_labels(decimal_places=1):
+            _decimal_places = str(decimal_places)
+
+            fmt = (
+                "[%." + _decimal_places + "f " +
+                ": %0." + _decimal_places + "f"
+            )
+
             labels = np.copy(self._info['bins'])
-            labels = ["[%.1f : %0.1f[" %(labels[i], labels[i+1]) \
-                      for i in range(len(labels)-1)]
+            if locale.getlocale()[0] in ['fr_FR']:
+                fmt += '['
+            else:
+                fmt += ')'
+
+            labels = [fmt % (labels[i], labels[i + 1])
+                      for i in range(len(labels) - 1)]
             return labels
 
         kwargs.pop('labels', None)
         kwargs.pop('handles', None)
+
+        decimal_places = kwargs.pop('decimal_places', 1)
+
         handles = get_handles()
-        labels = get_labels()
+        labels = get_labels(decimal_places)
         self.legend_ = mpl.legend.Legend(self, handles, labels, loc, **kwargs)
         return self.legend_
 
-    def set_legend(self):
-        l = self.legend(borderaxespad=-0.10)
+    def set_legend(self, **pyplot_arguments):
+        if 'borderaxespad' not in pyplot_arguments:
+            pyplot_arguments['borderaxespad'] = -0.10
+        l = self.legend(**pyplot_arguments)
         plt.setp(l.get_texts(), fontsize=8)
+        return l
 
     def _init_plot(self, direction, var, **kwargs):
         """
         Internal method used by all plotting commands
+        direction : 1D array - directions the wind blows from, North centred
+        var : 1D array - values of the variable to compute. Typically the wind
+              speeds
         """
-        #self.cla()
+
+        # if weibull factors are entered overwrite direction and var
+        if 'weibull_factors' in kwargs or 'mean_values' in kwargs:
+            if 'weibull_factors' in kwargs and 'mean_values' in kwargs:
+                raise TypeError("cannot specify both weibull_factors and mean_values")
+            statistic_type = 'unset'
+            if 'weibull_factors' in kwargs:
+                statistic_type = 'weibull'
+                val = kwargs.pop('weibull_factors')
+            elif 'mean_values' in kwargs:
+                statistic_type = 'mean'
+                val = kwargs.pop('mean_values')
+            if val:
+                if 'frequency' not in kwargs:
+                    raise TypeError("specify 'frequency' argument for statistical input")
+                windFrequencies = kwargs.pop('frequency')
+                if len(windFrequencies) != len(direction) or len(direction) != len(var):
+                    if len(windFrequencies) != len(direction):
+                        raise TypeError("len(frequency) != len(direction)")
+                    elif len(direction) != len(var):
+                        raise TypeError("len(frequency) != len(direction)")
+                windSpeeds = []
+                windDirections = []
+                for dbin in range(len(direction)):
+                    for _ in range(int(windFrequencies[dbin] * 10000)):
+                        if statistic_type == 'weibull':
+                            windSpeeds.append(random.weibullvariate(var[dbin][0], var[dbin][1]))
+                        elif statistic_type == 'mean':
+                            windSpeeds.append(random.weibullvariate(var[dbin] * 2 / np.sqrt(np.pi), 2))
+                        windDirections.append(direction[dbin])
+                var, direction = windSpeeds, windDirections
+
+        # self.cla()
         kwargs.pop('zorder', None)
 
-        #Init of the bins array if not set
+        # Init of the bins array if not set
         bins = kwargs.pop('bins', None)
         if bins is None:
             bins = np.linspace(np.min(var), np.max(var), 6)
@@ -198,17 +264,17 @@ class WindroseAxes(PolarAxes):
         bins = np.asarray(bins)
         nbins = len(bins)
 
-        #Number of sectors
+        # Number of sectors
         nsector = kwargs.pop('nsector', None)
         if nsector is None:
             nsector = 16
 
-        #Sets the colors table based on the colormap or the "colors" argument
+        # Sets the colors table based on the colormap or the "colors" argument
         colors = kwargs.pop('colors', None)
         cmap = kwargs.pop('cmap', None)
         if colors is not None:
             if isinstance(colors, str):
-                colors = [colors]*nbins
+                colors = [colors] * nbins
             if isinstance(colors, (tuple, list)):
                 if len(colors) != nbins:
                     raise ValueError("colors and bins must have same length")
@@ -217,18 +283,17 @@ class WindroseAxes(PolarAxes):
                 cmap = mpl.cm.jet
             colors = self._colors(cmap, nbins)
 
-        #Building the angles list
-        angles = np.arange(0, -2*np.pi, -2*np.pi/nsector) + np.pi/2
+        # Building the angles list
+        angles = np.arange(0, -2 * np.pi, -2 * np.pi / nsector) + np.pi / 2
 
         normed = kwargs.pop('normed', False)
         blowto = kwargs.pop('blowto', False)
 
-        #Set the global information dictionnary
+        # Set the global information dictionnary
         self._info['dir'], self._info['bins'], self._info['table'] = histogram(direction, var, bins, nsector, normed, blowto)
 
         return bins, nbins, nsector, colors, angles, kwargs
 
-
     def contour(self, direction, var, **kwargs):
         """
         Plot a windrose in linear mode. For each var bins, a line will be
@@ -260,19 +325,18 @@ class WindroseAxes(PolarAxes):
         others kwargs : see help(pylab.plot)
 
         """
-
         bins, nbins, nsector, colors, angles, kwargs = self._init_plot(direction, var,
                                                                        **kwargs)
 
-        #closing lines
-        angles = np.hstack((angles, angles[-1]-2*np.pi/nsector))
+        # closing lines
+        angles = np.hstack((angles, angles[-1] - 2 * np.pi / nsector))
         vals = np.hstack((self._info['table'],
-                         np.reshape(self._info['table'][:,0],
-                                   (self._info['table'].shape[0], 1))))
-        
+                         np.reshape(self._info['table'][:, 0],
+                                    (self._info['table'].shape[0], 1))))
+
         offset = 0
         for i in range(nbins):
-            val = vals[i,:] + offset
+            val = vals[i, :] + offset
             offset += vals[i, :]
             zorder = ZBASE + nbins - i
             patch = self.plot(angles, val, color=colors[i], zorder=zorder,
@@ -280,7 +344,6 @@ class WindroseAxes(PolarAxes):
             self.patches_list.extend(patch)
         self._update()
 
-
     def contourf(self, direction, var, **kwargs):
         """
         Plot a windrose in filled mode. For each var bins, a line will be
@@ -317,15 +380,15 @@ class WindroseAxes(PolarAxes):
                                                                        **kwargs)
         kwargs.pop('facecolor', None)
         kwargs.pop('edgecolor', None)
-        
-        #closing lines
-        angles = np.hstack((angles, angles[-1]-2*np.pi/nsector))
+
+        # closing lines
+        angles = np.hstack((angles, angles[-1] - 2 * np.pi / nsector))
         vals = np.hstack((self._info['table'],
-                         np.reshape(self._info['table'][:,0],
-                                   (self._info['table'].shape[0], 1))))
+                          np.reshape(self._info['table'][:, 0],
+                                     (self._info['table'].shape[0], 1))))
         offset = 0
         for i in range(nbins):
-            val = vals[i,:] + offset
+            val = vals[i, :] + offset
             offset += vals[i, :]
             zorder = ZBASE + nbins - i
             xs, ys = poly_between(angles, 0, val)
@@ -333,7 +396,6 @@ class WindroseAxes(PolarAxes):
                               edgecolor=colors[i], zorder=zorder, **kwargs)
             self.patches_list.extend(patch)
 
-
     def bar(self, direction, var, **kwargs):
         """
         Plot a windrose in bar mode. For each var bins and for each sector,
@@ -375,17 +437,18 @@ class WindroseAxes(PolarAxes):
         opening = kwargs.pop('opening', None)
         if opening is None:
             opening = 0.8
-        dtheta = 2*np.pi/nsector
-        opening = dtheta*opening
+        dtheta = 2 * np.pi / nsector
+        opening = dtheta * opening
 
         for j in range(nsector):
             offset = 0
             for i in range(nbins):
                 if i > 0:
-                    offset += self._info['table'][i-1, j]
+                    offset += self._info['table'][i - 1, j]
                 val = self._info['table'][i, j]
                 zorder = ZBASE + nbins - i
-                patch = mpl.patches.Rectangle((angles[j]-opening/2, offset), opening, val,
+                patch = mpl.patches.Rectangle(
+                    (angles[j] - opening / 2, offset), opening, val,
                     facecolor=colors[i], edgecolor=edgecolor, zorder=zorder,
                     **kwargs)
                 self.add_patch(patch)
@@ -393,7 +456,6 @@ class WindroseAxes(PolarAxes):
                     self.patches_list.append(patch)
         self._update()
 
-
     def box(self, direction, var, **kwargs):
         """
         Plot a windrose in proportional bar mode. For each var bins and for each
@@ -430,16 +492,17 @@ class WindroseAxes(PolarAxes):
         if edgecolor is not None:
             if not isinstance(edgecolor, str):
                 raise ValueError('edgecolor must be a string color')
-        opening = np.linspace(0.0, np.pi/16, nbins)
+        opening = np.linspace(0.0, np.pi / 16, nbins)
 
         for j in range(nsector):
             offset = 0
             for i in range(nbins):
                 if i > 0:
-                    offset += self._info['table'][i-1, j]
+                    offset += self._info['table'][i - 1, j]
                 val = self._info['table'][i, j]
                 zorder = ZBASE + nbins - i
-                patch = mpl.patches.Rectangle((angles[j]-opening[i]/2, offset), opening[i],
+                patch = mpl.patches.Rectangle(
+                    (angles[j] - opening[i] / 2, offset), opening[i],
                     val, facecolor=colors[i], edgecolor=edgecolor,
                     zorder=zorder, **kwargs)
                 self.add_patch(patch)
@@ -480,8 +543,8 @@ class WindAxes(mpl.axes.Subplot):
         self.bar(center, hist, align='center', width=width, color=bar_color)
         params = scipy.stats.exponweib.fit(var, floc=0, f0=1)
         x = np.linspace(0, bins[-1], Nx)
-        _ = self.plot(x, scipy.stats.exponweib.pdf(x, *params), color=plot_color)
-        return(self, params)        
+        self.plot(x, scipy.stats.exponweib.pdf(x, *params), color=plot_color)
+        return(self, params)
 
 
 def histogram(direction, var, bins, nsector, normed=False, blowto=False):
@@ -504,9 +567,9 @@ def histogram(direction, var, bins, nsector, normed=False, blowto=False):
     if len(var) != len(direction):
         raise(ValueError("var and direction must have same length"))
 
-    angle = 360./nsector
+    angle = 360. / nsector
 
-    dir_bins = np.arange(-angle/2 ,360.+angle, angle, dtype=np.float)
+    dir_bins = np.arange(-angle / 2, 360. + angle, angle, dtype=np.float)
     dir_edges = dir_bins.tolist()
     dir_edges.pop(-1)
     dir_edges[0] = dir_edges.pop(-1)
@@ -517,49 +580,49 @@ def histogram(direction, var, bins, nsector, normed=False, blowto=False):
 
     if blowto:
         direction = direction + 180.
-        direction[direction>=360.] = direction[direction>=360.] - 360
+        direction[direction >= 360.] = direction[direction >= 360.] - 360
 
     table = histogram2d(x=var, y=direction, bins=[var_bins, dir_bins],
-                          normed=False)[0]
+                        normed=False)[0]
     # add the last value to the first to have the table of North winds
-    table[:,0] = table[:,0] + table[:,-1]
+    table[:, 0] = table[:, 0] + table[:, -1]
     # and remove the last col
     table = table[:, :-1]
     if normed:
-        table = table*100/table.sum()
+        table = table * 100 / table.sum()
 
     return dir_edges, var_bins, table
 
 
-def wrcontour(direction, var, ax=None, **kwargs):
-    ax = WindroseAxes.from_ax(ax)
+def wrcontour(direction, var, ax=None, rmax=None, **kwargs):
+    ax = WindroseAxes.from_ax(ax, rmax=rmax)
     ax.contour(direction, var, **kwargs)
     ax.set_legend()
     return ax
 
 
-def wrcontourf(direction, var, ax=None, **kwargs):
-    ax = WindroseAxes.from_ax(ax)
+def wrcontourf(direction, var, ax=None, rmax=None, **kwargs):
+    ax = WindroseAxes.from_ax(ax, rmax=rmax)
     ax.contourf(direction, var, **kwargs)
     ax.set_legend()
     return ax
 
 
-def wrbox(direction, var, ax=None, **kwargs):
-    ax = WindroseAxes.from_ax(ax)
+def wrbox(direction, var, ax=None, rmax=None, **kwargs):
+    ax = WindroseAxes.from_ax(ax, rmax=rmax)
     ax.box(direction, var, **kwargs)
     ax.set_legend()
     return ax
 
 
-def wrbar(direction, var, ax=None, **kwargs):
-    ax = WindroseAxes.from_ax(ax)
+def wrbar(direction, var, ax=None, rmax=None, **kwargs):
+    ax = WindroseAxes.from_ax(ax, rmax=rmax)
     ax.bar(direction, var, **kwargs)
     ax.set_legend()
     return ax
 
 
-def wrpdf(var, bins=None, Nx=100, bar_color='b', plot_color='g', Nbins=10, ax=None, *args, **kwargs):
+def wrpdf(var, bins=None, Nx=100, bar_color='b', plot_color='g', Nbins=10, ax=None, rmax=None, *args, **kwargs):
     '''
     Draw probability density function
     and return Weitbull distribution parameters
@@ -569,24 +632,23 @@ def wrpdf(var, bins=None, Nx=100, bar_color='b', plot_color='g', Nbins=10, ax=No
     return(ax, params)
 
 
-def wrscatter(direction, var, ax=None, *args, **kwargs):
+def wrscatter(direction, var, ax=None, rmax=None, *args, **kwargs):
     '''
     Draw scatter plot
     '''
-    ax = WindroseAxes.from_ax(ax)
+    ax = WindroseAxes.from_ax(ax, rmax=rmax)
     ax.scatter(direction, var, *args, **kwargs)
     return ax
 
-
-#def clean(direction, var):
-#    '''
-#    Remove masked values in the two arrays, where if a direction data is masked,
-#    the var data will also be removed in the cleaning process (and vice-versa)
-#    '''
-#    dirmask = direction.mask==False
-#    varmask = direction.mask==False
-#    mask = dirmask*varmask
-#    return direction[mask], var[mask]
+# def clean(direction, var):
+#     '''
+#     Remove masked values in the two arrays, where if a direction data is masked,
+#     the var data will also be removed in the cleaning process (and vice-versa)
+#     '''
+#     dirmask = direction.mask==False
+#     varmask = direction.mask==False
+#     mask = dirmask*varmask
+#     return direction[mask], var[mask]
 
 
 def clean_df(df, var=VAR_DEFAULT, direction=DIR_DEFAULT):
@@ -596,7 +658,7 @@ def clean_df(df, var=VAR_DEFAULT, direction=DIR_DEFAULT):
     removed from DataFrame
     if a direction is nan, this row is also removed from DataFrame
     '''
-    return(df[df[var].notnull() & df[var]!=0 & df[direction].notnull()])
+    return(df[df[var].notnull() & df[var] != 0 & df[direction].notnull()])
 
 
 def clean(direction, var, index=False):
@@ -607,7 +669,7 @@ def clean(direction, var, index=False):
     if a direction is nan, data is also removed from both array
     '''
     dirmask = np.isfinite(direction)
-    varmask = (var!=0 & np.isfinite(var))
+    varmask = (var != 0 & np.isfinite(var))
     mask = dirmask * varmask
     if index is None:
         index = np.arange(mask.sum())
@@ -628,7 +690,8 @@ D_KIND_PLOT = {
     'scatter': wrscatter
 }
 
-def plot_windrose(direction_or_df, var=None, kind='contour', var_name=VAR_DEFAULT, direction_name=DIR_DEFAULT, **kwargs):
+
+def plot_windrose(direction_or_df, var=None, kind='contour', var_name=VAR_DEFAULT, direction_name=DIR_DEFAULT, by=None, rmax=None, **kwargs):
     if var is None:
         # Assuming direction_or_df is a DataFrame
         df = direction_or_df
@@ -636,25 +699,30 @@ def plot_windrose(direction_or_df, var=None, kind='contour', var_name=VAR_DEFAUL
         direction = df[direction_name].values
     else:
         direction = direction_or_df
-    return(plot_windrose_np(direction, var, kind=kind, **kwargs))
+    return(plot_windrose_np(direction, var, kind=kind, by=by, rmax=rmax, **kwargs))
 
-def plot_windrose_df(df, kind='contour', var_name=VAR_DEFAULT, direction_name=DIR_DEFAULT, **kwargs):
+
+def plot_windrose_df(df, kind='contour', var_name=VAR_DEFAULT, direction_name=DIR_DEFAULT, by=None, rmax=None, **kwargs):
     var = df[var_name].values
     direction = df[direction_name].values
-    return(plot_windrose_np(direction, var, **kwargs))
+    return(plot_windrose_np(direction, var, by=by, rmax=rmax, **kwargs))
 
-def plot_windrose_np(direction, var, kind='contour', clean_flag=True, **kwargs):
+
+def plot_windrose_np(direction, var, kind='contour', clean_flag=True, by=None, rmax=None, **kwargs):
     if kind in D_KIND_PLOT.keys():
         f_plot = D_KIND_PLOT[kind]
     else:
-        raise(Exception("kind=%r but it must be in %r" % (kind, d.keys())))
-    #if f_clean is not None:
-    #    df = f_clean(df)
-    #var = df[var_name].values
-    #direction = df[direction_name].values
+        raise(Exception("kind=%r but it must be in %r" % (kind, D_KIND_PLOT.keys())))
+    # if f_clean is not None:
+    #     df = f_clean(df)
+    # var = df[var_name].values
+    # direction = df[direction_name].values
     if clean_flag:
         var, direction = clean(var, direction)
-    ax = f_plot(direction=direction, var=var, **kwargs)
-    if kind not in ['pdf']:
-        ax.set_legend()
-    return ax
+    if by is None:
+        ax = f_plot(direction=direction, var=var, rmax=rmax, **kwargs)
+        if kind not in ['pdf']:
+            ax.set_legend()
+        return ax
+    else:
+        raise(NotImplementedError("'by' keyword not supported for now https://github.com/scls19fr/windrose/issues/10"))
